/*
 * @productionLimitsPanel.java Version 1.1 09/12/2013
 *
 * Copyright (c) 2013 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package production.gui;

import java.sql.Timestamp;
import java.sql.Date;
import java.text.SimpleDateFormat;
import java.text.ParseException;
import java.net.*;
import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import java.util.Observable;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;

/** Class productionLimitsPanel
 *  <p> This Class will create the Decline Curve Calculator Panel.
 *
 *  @version 1.1 09/12/2013
 *  @author  John R. Victorine
 */

public class productionLimitsPanel extends JPanel
{
  private static final int _START = 0;
  private static final int _END   = 1;

  private static final int YEAR    = 0;
  private static final int MONTH   = 1;
  private static final int DAY     = 2;
  private static final int HOUR    = 3;
  private static final int MIN     = 4;
  private static final int SEC     = 5;

  // Input Variables

  private Observable notifier = null; // Observalbles Object

  private String     sTitle1  = "Kansas Geological Survey";
  private String     sTitle2  = "CO2 Offset Gas";
  private String     sTitle3  = "";

  private Timestamp  tsStart  = null; // Starting Timestamp
  private Timestamp  tsEnd    = null; // Ending Timestamp

  // Global Variables

  private String     sTitle   = "";

  private int        iYear    = 0;
  private int        iMonth   = 0;

  private int        iYStart  = 0;
  private int        iYEnd    = 0;
  private int        iMStart  = 0;
  private int        iMEnd    = 0;

  // X-Axis Values.

  public int          iXLog       = 0;   // X-Axis Semilog Indicator
                                         //   1-Semilog
                                         //   0-Linear
  public int          iXCycles    = 1;   // Number of Cycles
  public double       dXMaximum   = 0.0; // Maximum X-Axis
  public double       dXMinimum   = 0.0; // Minimum X-Axis
  public double       dXIncrement = 0.0; // Increment along the X-Axis
  public String       sXAxis      = "";  // X-Axis Label.
  private String      sXIncr      = "Increment:";


  // Y-Axis Values.

  public int          iYLog       = 1;      // Y-Axis Semilog Indicator
                                            //   1-Semilog
                                            //   0-Linear
  public int          iYCycles    = 3;      // Number of Cycles
  public double       dYMaximum   = 1000.0; // Maximum Y-Axis
  public double       dYMinimum   = 1.0;    // Minimum Y-Axis
  public double       dYIncrement = 1.0;    // Increment along the Y-Axis
  public String       sYAxis      = "Production (MCF)";  // Y-Axis Label.
  private String      sYIncr      = "Cycles:";

  // Global Frame Widgets

  // .. Plot Titles

  private JTextField txtTitle1 = new JTextField();
  private JTextField txtTitle2 = new JTextField();
  private JTextField txtTitle3 = new JTextField();

  // .. X-Axis Widgets

  private JTextField txtXAxis  = new JTextField();

  private JTextField txtYYYYs  = new JTextField(); // Year Field
  private JTextField txtMMs    = new JTextField(); // Month Field
  private JTextField txtDDs    = new JTextField(); // Day Field

  private JTextField txtHHs    = new JTextField(); // 24 Hour Field
  private JTextField txtmms    = new JTextField(); // minute Field
  private JTextField txtssSs   = new JTextField(); // second Field

  private JTextField txtYYYYe  = new JTextField(); // Year Field
  private JTextField txtMMe    = new JTextField(); // Month Field
  private JTextField txtDDe    = new JTextField(); // Day Field

  private JTextField txtHHe    = new JTextField(); // 24 Hour Field
  private JTextField txtmme    = new JTextField(); // minute Field
  private JTextField txtssSe   = new JTextField(); // second Field

  // .. Y-Axis Widgets

  private JTextField txtYAxis  = new JTextField();
  private JTextField txtYMin   = new JTextField();
  private JTextField txtYMax   = new JTextField();
  private JTextField txtYIncr  = new JTextField();

  /** Constructor productionLimitsPanel()
   *  <p> This is the Constructor for this class.
   *  @param stList = Production Data List Structure
   */

  public productionLimitsPanel( Observable notifier,
	                            String sTitle1, String sTitle2, String sTitle3,
                                Timestamp  tsStart, Timestamp  tsEnd,
                                String sYAxis,
                                double dYmax, double dYmin, int iYCycles )
  {
    try
    {
	  this.notifier  = notifier;

	  this.sTitle1   = new String(sTitle1);
	  this.sTitle2   = new String(sTitle2);
	  this.sTitle3   = new String(sTitle3);

      this.tsStart   = new Timestamp(tsStart.getTime());
      this.tsEnd     = new Timestamp(tsEnd.getTime());

      this.sYAxis    = new String(sYAxis);
      this.iYCycles  = iYCycles;
      this.dYMaximum = dYmax;
      this.dYMinimum = dYmin;

      jbInit();
    }
    catch (Exception e)
    {
      e.printStackTrace();
    }
  }

  /** METHOD jbInit()
   *  <p> This Method will create the Frame for this class.
   *  @throws Exception
   */

  private void jbInit() throws Exception
  {
	JPanel      pnlTop      = new JPanel();
	JPanel      pnlTitles   = new JPanel();
	JPanel      pnlAxes     = new JPanel();
	JPanel      pnlXAxis    = new JPanel();
	JPanel      pnlYAxis    = new JPanel();

    this.setLayout(new BorderLayout());
    pnlTop.setLayout(new BorderLayout());

    // -- Titles

    pnlTitles = plotTitlesPanel();

    pnlAxes.setLayout(new BorderLayout());

    pnlXAxis = plotXAxisPanel();
    pnlYAxis = plotYAxisPanel();

    // Attach Widgets

    this.add(pnlTop,      BorderLayout.NORTH);
    pnlTop.add(pnlTitles, BorderLayout.CENTER);

    this.add(pnlAxes,     BorderLayout.SOUTH);
    pnlAxes.add(pnlXAxis, BorderLayout.CENTER);
    pnlAxes.add(pnlYAxis, BorderLayout.SOUTH);
  }

  /** Method plotTitlesPanel()
   *  <p> This method will build the Plot Title Panels
   */

  private JPanel plotTitlesPanel()
  {
	JPanel panel           = new JPanel();
    JPanel pnlTitles       = new JPanel();
    JPanel pnlTitle1       = new JPanel();
    JPanel pnlTitle2       = new JPanel();
    JPanel pnlTitle3       = new JPanel();

    TitledBorder titledBorderTitle1 = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "1st Title:");
    titledBorderTitle1.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderTitle2 = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "2nd Title:");
    titledBorderTitle2.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderTitle3 = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "3rd Title:");
    titledBorderTitle3.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    panel.setLayout(new BorderLayout());

    pnlTitles.setLayout(new GridLayout(3,1));

    // .. First Title Panel

    pnlTitle1.setLayout(new BorderLayout());
    pnlTitle1.setBorder(titledBorderTitle1);

    txtTitle1.setText(sTitle1);
    txtTitle1.setHorizontalAlignment(SwingConstants.LEFT);
    txtTitle1.addFocusListener(new productionLimitsPanelFocusAdapter(this));

    // .. Second Title Panel

    pnlTitle2.setLayout(new BorderLayout());
    pnlTitle2.setBorder(titledBorderTitle2);

    txtTitle2.setText(sTitle2);
    txtTitle2.setHorizontalAlignment(SwingConstants.LEFT);
    txtTitle2.addFocusListener(new productionLimitsPanelFocusAdapter(this));

    // .. Third Title Panel

    pnlTitle3.setLayout(new BorderLayout());
    pnlTitle3.setBorder(titledBorderTitle3);

    txtTitle3.setText(sTitle3);
    txtTitle3.setHorizontalAlignment(SwingConstants.LEFT);
    txtTitle3.addFocusListener(new productionLimitsPanelFocusAdapter(this));

    // Attach Widgets

    panel.add(pnlTitles,     BorderLayout.NORTH);
    pnlTitles.add(pnlTitle1, null);
    pnlTitle1.add(txtTitle1, BorderLayout.CENTER);

    pnlTitles.add(pnlTitle2, null);
    pnlTitle2.add(txtTitle2, BorderLayout.CENTER);

    pnlTitles.add(pnlTitle3, null);
    pnlTitle3.add(txtTitle3, BorderLayout.CENTER);

    return (panel);
  }

  /** Method plotXAxisPanel()
   *  <p> This method will build the X-Axis Panel
   */

  private JPanel plotXAxisPanel()
  {
	JPanel panel             = new JPanel();
    JPanel pnlXAxis          = new JPanel();

    JPanel pnlDateTimeCenter = new JPanel();

    JPanel pnlDateTimeStart  = new JPanel();
    JPanel pnlDateTimeEnd    = new JPanel();

    JPanel pnlDateStart      = new JPanel();
    JPanel pnlYYYYs          = new JPanel();
    JPanel pnlMMs            = new JPanel();
    JPanel pnlDDs            = new JPanel();

    JPanel pnlTimeStart      = new JPanel();
    JPanel pnlHHs            = new JPanel();
    JPanel pnlmms            = new JPanel();
    JPanel pnlsss            = new JPanel();

    JPanel pnlDateEnd        = new JPanel();
    JPanel pnlYYYYe          = new JPanel();
    JPanel pnlMMe            = new JPanel();
    JPanel pnlDDe            = new JPanel();

    JPanel pnlTimeEnd        = new JPanel();
    JPanel pnlHHe            = new JPanel();
    JPanel pnlmme            = new JPanel();
    JPanel pnlsse            = new JPanel();

	String sText             = "";

    TitledBorder titledBorderXAxis = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "X-Axis:");
    titledBorderXAxis.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderStartDate = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Start Date");
    titledBorderStartDate.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderEndDate = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "End Date");
    titledBorderEndDate.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderYYYY = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "YYYY");
    titledBorderYYYY.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderMM = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "MM");
    titledBorderYYYY.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderDD = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "DD");
    titledBorderYYYY.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderHH = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "HH");
    titledBorderHH.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBordermm = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "mm");
    titledBordermm.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderssS = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "ss.S");
    titledBorderssS.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    panel.setLayout(new BorderLayout());

    // .. X-Axis Panel

    pnlXAxis.setLayout(new GridLayout(2,1));
    pnlXAxis.setBorder(titledBorderXAxis);

    // .......................................................
    // ..................... Date Start ......................
    // .......................................................

    iYear  = 0;
    iMonth = 0;

    if (tsStart != null)
    {
	  setDate(tsStart);
	}

    pnlDateTimeStart.setLayout(new GridLayout());
    pnlDateTimeStart.setBorder(titledBorderStartDate);

    // -- Date Start Panel

    pnlDateStart.setLayout(new BorderLayout());

    // .. Year Textfield Panel

    pnlYYYYs.setLayout(new BorderLayout());
    pnlYYYYs.setBorder(titledBorderYYYY);
    pnlYYYYs.add(txtYYYYs,  BorderLayout.CENTER);

    sText = new String("2014");
    if (iYear > 0) { sText = new String(""+iYear); }

    txtYYYYs.setText( sText );
    txtYYYYs.setColumns(4);
	txtYYYYs.setHorizontalAlignment(SwingConstants.TRAILING);
	txtYYYYs.addFocusListener(new productionLimitsPanelFocusAdapter(this));

    // .. Month Textfield Panel

    pnlMMs.setLayout(new BorderLayout());
    pnlMMs.setBorder(titledBorderMM);
    pnlMMs.add(txtMMs,  BorderLayout.CENTER);

    sText = new String("01");
    if (iMonth > 0)
    {
	  if (iMonth > 9)
	    sText = new String("" + iMonth);
	  else
	    sText = new String("0" + iMonth);
    }

    txtMMs.setText( sText );
    txtMMs.setColumns(2);
	txtMMs.setHorizontalAlignment(SwingConstants.TRAILING);
	txtMMs.addFocusListener(new productionLimitsPanelFocusAdapter(this));

    // .. Day Textfield Panel

    pnlDDs.setLayout(new BorderLayout());
    pnlDDs.setBorder(titledBorderDD);
    pnlDDs.add(txtDDs,  BorderLayout.CENTER);

    txtDDs.setText("01");
    txtDDs.setEditable(false);
    txtDDs.setColumns(2);
	txtDDs.setHorizontalAlignment(SwingConstants.TRAILING);
	txtDDs.addFocusListener(new productionLimitsPanelFocusAdapter(this));

    // -- Attach YYYY,MM,DD Panels to Date Start Panel

    pnlDateStart.add(pnlYYYYs, BorderLayout.WEST);
    pnlDateStart.add(pnlMMs,   BorderLayout.CENTER);
    pnlDateStart.add(pnlDDs,   BorderLayout.EAST);

    // -- Time Start Panel

    pnlTimeStart.setLayout(new BorderLayout());

    // .. Hour Textfield Panel

    pnlHHs.setLayout(new BorderLayout());
    pnlHHs.setBorder(titledBorderHH);
    pnlHHs.add(txtHHs, BorderLayout.CENTER);

    txtHHs.setText("00");
    txtHHs.setEditable(false);
    txtHHs.setColumns(2);
	txtHHs.setHorizontalAlignment(SwingConstants.TRAILING);
	txtHHs.addFocusListener(new productionLimitsPanelFocusAdapter(this));

    // .. Minutes Textfield Panel

    pnlmms.setLayout(new BorderLayout());
    pnlmms.setBorder(titledBordermm);
    pnlmms.add(txtmms, BorderLayout.CENTER);

    txtmms.setText("00");
    txtmms.setEditable(false);
    txtmms.setColumns(2);
	txtmms.setHorizontalAlignment(SwingConstants.TRAILING);
	txtmms.addFocusListener(new productionLimitsPanelFocusAdapter(this));

    // .. Seconds Textfield Panel

    pnlsss.setLayout(new BorderLayout());
    pnlsss.setBorder(titledBorderssS);
    pnlsss.add(txtssSs, BorderLayout.CENTER);

    txtssSs.setText("00.0");
    txtssSs.setEditable(false);
    txtssSs.setColumns(4);
	txtssSs.setHorizontalAlignment(SwingConstants.TRAILING);
	txtssSs.addFocusListener(new productionLimitsPanelFocusAdapter(this));

    // -- Attach HH,mm,ss.SS Panels to Date Start Panel

    pnlTimeStart.add(pnlHHs, BorderLayout.WEST);
    pnlTimeStart.add(pnlmms, BorderLayout.CENTER);
    pnlTimeStart.add(pnlsss, BorderLayout.EAST);

    tsStart = getTimestamp( _START );

    // .......................................................
    // ...................... Date End .......................
    // .......................................................

    iYear  = 0;
    iMonth = 0;

    if (tsEnd != null)
    {
	  setDate(tsEnd);
	}

    pnlDateTimeEnd.setLayout(new GridLayout());
    pnlDateTimeEnd.setBorder(titledBorderEndDate);

    // -- Date End Panel

    pnlDateEnd.setLayout(new BorderLayout());

    // .. Year Textfield Panel

    pnlYYYYe.setLayout(new BorderLayout());
    pnlYYYYe.setBorder(titledBorderYYYY);
    pnlYYYYe.add(txtYYYYe,  BorderLayout.CENTER);

    sText = new String("2015");
    if (iYear > 0) { sText = new String(""+iYear); }

    txtYYYYe.setText( sText );
    txtYYYYe.setColumns(4);
	txtYYYYe.setHorizontalAlignment(SwingConstants.TRAILING);
	txtYYYYe.addFocusListener(new productionLimitsPanelFocusAdapter(this));

    // .. Month Textfield Panel

    pnlMMe.setLayout(new BorderLayout());
    pnlMMe.setBorder(titledBorderMM);
    pnlMMe.add(txtMMe,  BorderLayout.CENTER);

    sText = new String("12");
    if (iMonth > 0)
    {
//	  iMonth++;
	  if (iMonth > 9)
	    sText = new String("" + iMonth);
	  else
	    sText = new String("0" + iMonth);
    }

    txtMMe.setText( sText );
    txtMMe.setColumns(2);
	txtMMe.setHorizontalAlignment(SwingConstants.TRAILING);
	txtMMe.addFocusListener(new productionLimitsPanelFocusAdapter(this));

    // .. Day Textfield Panel

    pnlDDe.setLayout(new BorderLayout());
    pnlDDe.setBorder(titledBorderDD);
    pnlDDe.add(txtDDe,  BorderLayout.CENTER);

    txtDDe.setText("01");
    txtDDe.setEditable(false);
    txtDDe.setColumns(2);
	txtDDe.setHorizontalAlignment(SwingConstants.TRAILING);
	txtDDe.addFocusListener(new productionLimitsPanelFocusAdapter(this));

    // -- Attach YYYY,MM,DD Panels to Date Start Panel

    pnlDateEnd.add(pnlYYYYe, BorderLayout.WEST);
    pnlDateEnd.add(pnlMMe,   BorderLayout.CENTER);
    pnlDateEnd.add(pnlDDe,   BorderLayout.EAST);

    // -- Time End Panel

    pnlTimeEnd.setLayout(new BorderLayout());

    // .. Hour Textfield Panel

    pnlHHe.setLayout(new BorderLayout());
    pnlHHe.setBorder(titledBorderHH);
    pnlHHe.add(txtHHe, BorderLayout.CENTER);

    txtHHe.setText("00");
    txtHHe.setEditable(false);
    txtHHe.setColumns(2);
	txtHHe.setHorizontalAlignment(SwingConstants.TRAILING);
	txtHHe.addFocusListener(new productionLimitsPanelFocusAdapter(this));

    // .. Minutes Textfield Panel

    pnlmme.setLayout(new BorderLayout());
    pnlmme.setBorder(titledBordermm);
    pnlmme.add(txtmme, BorderLayout.CENTER);

    txtmme.setText("00");
    txtmme.setEditable(false);
    txtmme.setColumns(2);
	txtmme.setHorizontalAlignment(SwingConstants.TRAILING);
	txtmme.addFocusListener(new productionLimitsPanelFocusAdapter(this));

    // .. Seconds Textfield Panel

    pnlsse.setLayout(new BorderLayout());
    pnlsse.setBorder(titledBorderssS);
    pnlsse.add(txtssSe, BorderLayout.CENTER);

    txtssSe.setText("00.0");
    txtssSe.setEditable(false);
    txtssSe.setColumns(4);
	txtssSe.setHorizontalAlignment(SwingConstants.TRAILING);
	txtssSe.addFocusListener(new productionLimitsPanelFocusAdapter(this));

    // -- Attach HH,mm,ss.SS Panels to Date Start Panel

    pnlTimeEnd.add(pnlHHe, BorderLayout.WEST);
    pnlTimeEnd.add(pnlmme, BorderLayout.CENTER);
    pnlTimeEnd.add(pnlsse, BorderLayout.EAST);

    tsEnd = getTimestamp( _END );

    // Attach Widgets

    panel.add(pnlXAxis,                BorderLayout.CENTER);

    pnlXAxis.add(pnlDateTimeStart,     null);
    pnlDateTimeStart.add(pnlDateStart, null);
    pnlDateTimeStart.add(pnlTimeStart, null);

    pnlXAxis.add(pnlDateTimeEnd,       null);
    pnlDateTimeEnd.add(pnlDateEnd,     null);
    pnlDateTimeEnd.add(pnlTimeEnd,     null);

    return (panel);
  }

  /** Method plotYAxisPanel()
   *  <p> This method will build the Y-Axis Panel
   */

  private JPanel plotYAxisPanel()
  {
	JPanel panel           = new JPanel();
    JPanel pnlYAxis        = new JPanel();
    JPanel pnlYAxisRange   = new JPanel();
    JPanel pnlYMin         = new JPanel();
    JPanel pnlYMax         = new JPanel();
    JPanel pnlYIncr        = new JPanel();

    JLabel lblYAxis        = new JLabel();
    JLabel lblYMin         = new JLabel();
    JLabel lblYMax         = new JLabel();
    JLabel lblYIncr        = new JLabel();

    TitledBorder titledBorderYAxis = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Y-Axis:");
    titledBorderYAxis.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderMin = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Minimum:");
    titledBorderMin.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderMax = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Maximum:");
    titledBorderMax.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderXIncr = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Cycles");
    titledBorderXIncr.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    panel.setLayout(new BorderLayout());

    // .. Y-Axis Panel

    pnlYAxis.setLayout(new BorderLayout());
    pnlYAxis.setBorder(titledBorderYAxis);

    txtYAxis.setText( sYAxis );
    txtYAxis.setHorizontalAlignment(SwingConstants.LEFT);
    txtYAxis.addFocusListener(new productionLimitsPanelFocusAdapter(this));

    // .. Y-Axis Range Panel

    pnlYAxisRange.setLayout(new GridLayout());

    // .. .. Y-Axis Minimum Panel

    pnlYMin.setLayout(new BorderLayout());
    pnlYMin.setBorder(titledBorderMin);

    lblYMin.setText("Maximum: ");

    txtYMin.setText("" + dYMinimum);
    txtYMin.setColumns(6);
    txtYMin.setHorizontalAlignment(SwingConstants.RIGHT);
    txtYMin.addFocusListener(new productionLimitsPanelFocusAdapter(this));

    // .. .. Y-Axis Maximum Panel

    pnlYMax.setLayout(new BorderLayout());
    pnlYMax.setBorder(titledBorderMax);

    lblYMax.setText(" Maximum: ");

    txtYMax.setEditable(false);
    txtYMax.setText("" + dYMaximum);
    txtYMax.setColumns(6);
    txtYMax.setHorizontalAlignment(SwingConstants.RIGHT);
    txtYMax.addFocusListener(new productionLimitsPanelFocusAdapter(this));

    // .. .. Y-Axis Increment Panel

    pnlYIncr.setLayout(new BorderLayout());
    pnlYIncr.setBorder(titledBorderXIncr);

    lblYIncr.setText(" " + sYIncr +": ");

    txtYIncr.setText("" + iYCycles);
    txtYIncr.setColumns(6);
    txtYIncr.setHorizontalAlignment(SwingConstants.RIGHT);
    txtYIncr.addFocusListener(new productionLimitsPanelFocusAdapter(this));

    // Attach Widgets to Panel

    panel.add(pnlYAxis,         BorderLayout.CENTER);
    pnlYAxis.add(txtYAxis,      BorderLayout.NORTH);

    pnlYAxis.add(pnlYAxisRange, BorderLayout.SOUTH);
    pnlYAxisRange.add(pnlYMin,  null);
    pnlYMin.add(txtYMin,        BorderLayout.CENTER);

    pnlYAxisRange.add(pnlYMax,  null);
    pnlYMax.add(txtYMax,        BorderLayout.CENTER);

    pnlYAxisRange.add(pnlYIncr, null);
    pnlYIncr.add(txtYIncr,      BorderLayout.CENTER);

    return (panel);
  }

  /** Method close()
   * <p> This method will close all dialogs that are opened by this class.
   */

  public void close()
  {
    notifier   = null; // Observables

    tsStart   = null; // Starting Timestamp
    tsEnd     = null; // Ending Timestamp

    sTitle    = null;
    sXAxis    = null;
    sXIncr    = null;
    sYAxis    = null;
    sYIncr    = null;

    // Global Frame Widgets

    // .. Plot Titles

    txtTitle1 = null;
    txtTitle2 = null;
    txtTitle3 = null;

    // .. X-Axis Widgets

    txtXAxis  = null;
//    txtXMin   = null;
//    txtXMax   = null;
//    txtXIncr  = null;

    // .. Y-Axis Widgets

    txtYAxis  = null;
    txtYMin   = null;
    txtYMax   = null;
    txtYIncr  = null;

    // .. Start Date & Time

    txtYYYYs    = null; // Year Field
    txtMMs      = null; // Month Field
    txtDDs      = null; // Day Field

    txtHHs      = null; // 24 Hour Field
    txtmms      = null; // minute Field
    txtssSs     = null; // second Field

    // .. End Date & Time

    txtYYYYe    = null; // Year Field
    txtMMe      = null; // Month Field
    txtDDe      = null; // Day Field

    txtHHe      = null; // 24 Hour Field
    txtmme      = null; // minute Field
    txtssSe     = null; // second Field
  }

  /* ===================================================================== *
   * -------------------------- GET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method getTimestamp()
   *  <p> This method will convert the Date Time String into a Timestamp.
   *  @param iDate = Start (0) or End (1)
   */

  private Timestamp getTimestamp( int iDate )
  {
	Timestamp        ts    = null;
    SimpleDateFormat dfm   = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.SSSSSS");
    String           sText = "";
    long             ltime = (long) 0;

    switch (iDate)
    {
      case _START:
        sText   = new String( txtYYYYs.getText() + "-" +
		                      txtMMs.getText()   + "-" +
		                      txtDDs.getText()   + " " +
		                      txtHHs.getText()   + ":" +
		                      txtmms.getText()   + ":" +
		                      txtssSs.getText() );
        break;

      case _END:
        sText   = new String( txtYYYYe.getText() + "-" +
  		                      txtMMe.getText()   + "-" +
		                      txtDDe.getText()   + " " +
		                      txtHHe.getText()   + ":" +
		                      txtmme.getText()   + ":" +
		                      txtssSe.getText() );
        break;
    }

    try
    {
      ltime = dfm.parse(sText).getTime();
      ts    = new Timestamp( ltime );
	}
    catch (ParseException e)
    {
      e.printStackTrace();
    }

	return (ts);
  }

  public String getTitle(int iNo)
  {
	String sTitle = "";

    switch (iNo)
    {
      case 1:
        sTitle = new String( sTitle1 );   // First line of the plot title
        break;

      case 2:
        sTitle = new String( sTitle2 );   // Second line of the plot title
        break;

      case 3:
        sTitle = new String( sTitle3 );   // Third line of the plot title
        break;
	}

    return (sTitle);
  }

  public Timestamp getTime(int iNo)
  {
	Timestamp ts = null;

	switch (iNo)
	{
	  case _START:
	    ts = new Timestamp(tsStart.getTime());
	    break;
	  case _END:
	    ts = new Timestamp(tsEnd.getTime());
	    break;
    }

    return (ts);
  }

  public String getYAxis()  { return (sYAxis); }    // Y-Axis Label
  public double getYMax()   { return (dYMaximum); } // Maximum Y-Axis
  public double getYMin()   { return (dYMinimum); } // Minimum Y-Axis
  public int    getYCycle() { return (iYCycles); }  // Number of Log Cycles (Only of Log Scales)

  /* ===================================================================== *
   * -------------------------- SET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method setDate()
   *  <p> This method will set the Date Value into either the
   *      Start Date or End Date.
   *  @param ts    = timestamp
   */

  private void setDate(Timestamp ts)
  {
    int i = 0;
    String tokens[]    = null;
    String sDelimiter  = new String("[ ]+");
    String sDelimiter1 = new String("[-]+");
    String str         = "";
    String sDate       = "";
    String date[]      = { "", "", "" };
    int    iContinue   = 0;
    double d           = 0.0;
    int    id          = 0;

    if (ts != null)
    {
      str = new String(ts.toString());
      tokens = str.split( sDelimiter );
      if (tokens.length > 0)
        sDate = new String(tokens[0].trim());

      tokens = sDate.split( sDelimiter1 );
      if (tokens.length == 3)
      {
  	    for (i=0; i<3; i++)
	    {
	      date[i] = new String(tokens[i]);
	    }

        iYear  = cmn.cmnString.stringToInt(date[0]);
	    iMonth = cmn.cmnString.stringToInt(date[1]);
      }
    }
  }

  /* ===================================================================== *
   * ---------------------------- ACTIONS -------------------------------- *
   * ===================================================================== */

  /** METHOD focusLost()
   *  <p> This Method will handle the lost focus event for the text fields.
   *  @param e      = Focus Lost Event
   */

  protected void focusLost(FocusEvent e)
  {
	focusLostTitles(e);
	focusLostXAxes(e);
	focusLostYAxes(e);
  }

  /** METHOD focusLostTitles()
   *  <p> This Method will handle the lost focus event for the text fields.
   *  @param e      = Focus Lost Event
   */

  protected void focusLostTitles(FocusEvent e)
  {
    if (e.getSource() == txtTitle1)
    {
      if (notifier != null)
        notifier.notifyObservers(new String("GET TITLE 1"));
      sTitle1 = new String(txtTitle1.getText());
//	  setTitle1(txtTitle1.getText());
	}

    if (e.getSource() == txtTitle2)
    {
      if (notifier != null)
        notifier.notifyObservers(new String("GET TITLE 2"));
      sTitle2 = new String(txtTitle2.getText());
//	  setTitle2(txtTitle2.getText());
	}

    if (e.getSource() == txtTitle3)
    {
      if (notifier != null)
        notifier.notifyObservers(new String("GET TITLE 3"));
      sTitle3 = new String(txtTitle3.getText());
//	  setTitle3(txtTitle3.getText());
	}
  }

  /** METHOD focusLostXAxes()
   *  <p> This Method will handle the lost focus event for the text fields.
   *  @param e      = Focus Lost Event
   */

  protected void focusLostXAxes(FocusEvent e)
  {
    int     iNumeric = 0;
    String  sMessage = new String("");
    String  sTemp    = new String("");
    int     iField   = -1;
    int     iDate    = -1;
    int     id       = 0;
    int     len      = 0;
    boolean bVal     = true;

    if ((e.getSource() == txtYYYYs) ||
        (e.getSource() == txtMMs )  ||
        (e.getSource() == txtYYYYe) ||
        (e.getSource() == txtMMe))
    {
      if (e.getSource() == txtYYYYs )
      {
        iNumeric = 1;
        iDate    = _START;
        iField   = YEAR;
        sTemp    = txtYYYYs.getText();
        len      = sTemp.length();
        sMessage = new String("Start Year Value is a Numeric Field");
	  }

      if (e.getSource() == txtYYYYe )
      {
        iNumeric = 1;
        iDate    = _END;
        iField   = YEAR;
        sTemp    = txtYYYYe.getText();
        len      = sTemp.length();
        sMessage = new String("End Year Value is a Numeric Field");
	  }

      if (e.getSource() == txtMMs )
      {
        iNumeric = 1;
        iDate    = _START;
        iField   = MONTH;
        sTemp    = txtMMs.getText();
        len      = sTemp.length();
        sMessage = new String("Start Month Value is a Numeric Field");
	  }

      if (e.getSource() == txtMMe )
      {
        iNumeric = 1;
        iDate    = _END;
        iField   = MONTH;
        sTemp    = txtMMe.getText();
        len      = sTemp.length();
        sMessage = new String("End Month Value is a Numeric Field");
	  }

      if (iNumeric == 1)
      {
        if (!cmn.cmnString.isNumeric(sTemp))
        {
          JOptionPane.showMessageDialog((Component) null,
                                         sMessage,
                                         "ERROR",
                                         JOptionPane.ERROR_MESSAGE);
        }
        else
        {
		  id = cmn.cmnString.stringToInt(sTemp);

          switch (iDate)
          {
            case _START:
              switch (iField)
              {
                case YEAR:
                  if (len != 4) { bVal = false; }
                  break;
                case MONTH:
                  if (len > 2) { bVal = false; }
                  else if ((id < 0) || (id > 12)) { bVal = false; }
                  else if (len < 2) { txtMMs.setText("0"+sTemp); }
                  break;
	          }
              break;

            case _END:
  	          switch (iField)
              {
                case YEAR:
                  if (len != 4) { bVal = false; }
                  break;
                case MONTH:
                  if (len > 2) { bVal = false; }
                  else if ((id < 0) || (id > 12)) { bVal = false; }
                  else if (len < 2) { txtMMe.setText("0"+sTemp); }
                  break;
	          }
	          break;
		  }
		}
	  }

	  if (bVal)
	  {
        tsStart = getTimestamp( _START );
        tsEnd   = getTimestamp( _END );
        if (notifier != null)
          notifier.notifyObservers(new String("GET X"));
//        setXAxis( tsStart, tsEnd );
	  }
	}
  }

  /** METHOD focusLostYAxes()
   *  <p> This Method will handle the lost focus event for the text fields.
   *  @param e      = Focus Lost Event
   */

  protected void focusLostYAxes(FocusEvent e)
  {
    int    iNumeric = 0;
    String sMessage = new String("");
    String sTemp    = new String("");
    String s1       = "";
    String s2       = "";
    String s3       = "";
    double d1       = 0.0;
    double d2       = 0.0;
    double dTemp    = 0.0;
    int    iMin     = 0;
    int    iMax     = 0;
    int    iDiff    = 0;
    double dMin     = 0.0;
    double dMax     = 0.0;
    double diff     = 0.0;

    if ((e.getSource() == txtYAxis) ||
        (e.getSource() == txtYMin ) ||
        (e.getSource() == txtYMax ) ||
        (e.getSource() == txtYIncr))
    {
      if (e.getSource() == txtYAxis)
      {
        sYAxis = new String( txtYAxis.getText() );
	    if (notifier != null)
		  notifier.notifyObservers(new String("GET Y"));
//        setYAxis( sYAxis, iYLog, dYMaximum, dYMinimum, dYIncrement, iYCycles );
	  }

      if (e.getSource() == txtYMin )
      {
        iNumeric = 1;
        sTemp    = txtYMin.getText();
        sMessage = new String("Y-Minimum Value is a Numeric Field");

        s1 = txtYMin.getText();
        s2 = txtYMax.getText();

        if (s1.equals(s2))
        {
	      txtYMin.setText(""+dYMinimum);

	      sTemp = new String("A");
          sMessage = new String("Y-Minimum Value can not equal Y-Maximum Value");
	    }

	    if ((cmn.cmnString.isNumeric(s1)) && (cmn.cmnString.isNumeric(s2)))
	    {
	      d1 = cmn.cmnString.stringToDouble( s1 );
	      d2 = cmn.cmnString.stringToDouble( s2 );

	      if (d1 > d2 )
	      {
	     	  txtYMin.setText(""+dYMinimum);

	    	  sTemp = new String("A");
              sMessage = new String("Y-Minimum Value can not be greater than Y-Maximum Value");
	       }
	    }
	  }

      if (e.getSource() == txtYMax )
      {
        iNumeric = 1;
        sTemp    = txtYMax.getText();
        sMessage = new String("Y-Maximum Value is a Numeric Field");

        s1 = txtYMin.getText();
        s2 = txtYMax.getText();

        if (s1.equals(s2))
        {
	      txtYMax.setText(""+dYMaximum);

	      sTemp = new String("A");
          sMessage = new String("Y-Minimum Value can not equal Y-Maximum Value");
	    }

	    if ((cmn.cmnString.isNumeric(s1)) && (cmn.cmnString.isNumeric(s2)))
	    {
	      d1 = cmn.cmnString.stringToDouble( s1 );
	      d2 = cmn.cmnString.stringToDouble( s2 );

	      if (d1 > d2 )
	      {
	        txtYMax.setText(""+dYMaximum);

	        sTemp = new String("A");
            sMessage = new String("Y-Minimum Value can not be greater than Y-Maximum Value");
	      }
	    }
	  }

      if (e.getSource() == txtYIncr)
      {
        iNumeric = 1;
        sTemp    = txtYIncr.getText();
        sMessage = new String("Y-Increment Value is a Numeric Field");

        s3 = txtYIncr.getText();
	    if ((s3.equals("0.0")) || (s3.equals("0")))
	    {
	   	  txtYIncr.setText(""+dYIncrement);
	      if (iYLog == 1) { txtYIncr.setText(""+iYCycles); }
	   	  sTemp = new String("A");
          sMessage = new String("Y-Increment Value can not be 0");
	    }
	  }

      if (iNumeric == 1)
      {
        if (!cmn.cmnString.isNumeric(sTemp))
        {
          JOptionPane.showMessageDialog((Component) null,
                                         sMessage,
                                         "ERROR",
                                         JOptionPane.ERROR_MESSAGE);
        }
        else
        {
          if ((e.getSource() == txtYMin ) ||
              (e.getSource() == txtYMax ) ||
              (e.getSource() == txtYIncr ))
          {
            if (e.getSource() == txtYMin )
            {
              dYMinimum = cmn.cmnString.stringToDouble(txtYMin.getText());

	   	      if (iYLog == 1)
	 	      {
                dMin = Math.log(dYMinimum)/Math.log(10.0);
                if (dMin < 0.0) { dMin = dMin - 1.0; }
                iMin = (int) dMin;
                dYMinimum = Math.pow(10.0, iMin);
                txtYMin.setText(""+dYMinimum);

                iMax = iMin + iYCycles;
                dYMaximum = Math.pow(10.0, iMax);
                txtYMax.setText(""+dYMaximum);
	          }
	        }

            if (e.getSource() == txtYMax )
              dYMaximum = cmn.cmnString.stringToDouble(txtYMax.getText());

            if (e.getSource() == txtYIncr )
            {
	          dTemp = cmn.cmnString.stringToDouble(txtYIncr.getText());

		      if (iYLog == 0)
		        dYIncrement = dTemp;
		      else
		      {
		        iYCycles    = (int) dTemp;
                dMin = Math.log(dYMinimum)/Math.log(10.0);
                if (dMin < 0.0) { dMin = dMin - 1.0; }
                iMin = (int) dMin;
                iMax = iMin + iYCycles;
                dYMaximum = Math.pow(10.0, iMax);
                txtYMax.setText(""+dYMaximum);
//		        txtYIncr.setText(""+iYCycles);
	          }
/*
		      if (iYLog == 1)
		      {
                iMin = (int) ( Math.log(dYMinimum)/Math.log(10.0) );
                iMax = iMin + iYCycles;
                dYMaximum = Math.pow(10.0, iMax);
                txtYMax.setText(""+dYMaximum);
	          }
*/
	        }

  	        if (notifier != null)
		      notifier.notifyObservers(new String("GET Y"));

//            setYAxis( sYAxis, iYLog, dYMaximum, dYMinimum, dYIncrement, iYCycles );
		  }
        }
	  }
	}
  }
}

/** CLASS productionLimitsPanelFocusAdapter()
  *  <p> This Class will handle Actions when focus is lost on a textfield
  */

class productionLimitsPanelFocusAdapter extends java.awt.event.FocusAdapter
{
  productionLimitsPanel adaptee;

  productionLimitsPanelFocusAdapter(productionLimitsPanel adaptee)
  {
    this.adaptee = adaptee;
  }

  public void focusLost(FocusEvent e) { adaptee.focusLost(e); }
}

/*
 *  @version 1.1 09/12/2013
 *  @author  John Victorine
 */
