/*
 * @productionPlotFrame.java Version 1.1 03/31/206
 *
 * Copyright (c) 2015 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package production.gui;

import java.util.Observer;
import java.util.Observable;
import java.awt.image.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;
import java.sql.Timestamp;
import java.sql.Date;
import java.text.SimpleDateFormat;
import java.text.ParseException;

import gui.guiSearchDirectoryFrame;           // File Search Dialog

import production.plot.productionPlot;        // Production Plot
import production.productionPlotStruct;       // Production Plot Data Structure
import production.productionListStruct;       // Production List Structure
import production.gui.productionLimitsPanel;  // Limits Calculator Panel

//import plot.plotSymbolStruct;

/** Class productionPlotFrame
 *  <p> This class will display the 2D Plot Frame
 *
 *  @version 1.1 03/31/206
 *  @author  John R. Victorine
 */

public class productionPlotFrame extends JFrame implements ActionListener, Observer
{
  private static final int OFF    = cmn.cmnStruct.OFF;
  private static final int ON     = cmn.cmnStruct.ON;

  private static final int _START = 0;
  private static final int _END   = 1;

  private static final int YEAR   = 0;
  private static final int MONTH  = 1;
  private static final int DAY    = 2;
  private static final int HOUR   = 3;
  private static final int MIN    = 4;
  private static final int SEC    = 5;

  // Input Variables

  private Observable            notifier = null; // Observalbles Object
  private int                   iPlot    = -1;
  private productionPlotStruct  stData   = null;
  private productionPlot        panel    = null;

  // Global Variables

  // -- Data Type

  private int                   iTypes    = 0;
  private String                sTypes[]  = { "", "", "", "" };
  private productionPlotStruct  stDatas[] = null;

  private int                   iData    = 0;

  // -- Set Limits Panel

  private productionLimitsPanel pLimits   = null; // Production Limits Panel

  // -- Save Path & Filename

  private String                  sPath   = new String("");
  private String                  sName   = new String("");
  private guiSearchDirectoryFrame pSearch = null; // Search for Directory to Save PNG

  // -- Time Variables

  private Timestamp               tsStart = null; // Starting Timestamp
  private Timestamp               tsEnd   = null; // Ending Timestamp

  private int                     iYear   = 0;
  private int                     iMonth  = 0;

  // -- Titles Variables

  private String                  sTitle  = "";

  private String                  sTitle1 = "Kansas Geological Survey";
  private String                  sTitle2 = "Fluid Level & CO2 Offset Gas";
  private String                  sTitle3 = "";

  // X-Axis Values.

  public int          iXLog       = 0;   // X-Axis Semilog Indicator
                                         //   1-Semilog
                                         //   0-Linear
  public int          iXCycles    = 1;   // Number of Cycles
  public double       dXMaximum   = 0.0; // Maximum X-Axis
  public double       dXMinimum   = 0.0; // Minimum X-Axis
  public double       dXIncrement = 0.0; // Increment along the X-Axis
  public String       sXAxis      = "";  // X-Axis Label.
  private String      sXIncr      = "Increment:";

  // Y-Axis Values.

  public int          iYLog       = 1;      // Y-Axis Semilog Indicator
                                            //   1-Semilog
                                            //   0-Linear
  public int          iYCycles    = 3;      // Number of Cycles
  public double       dYMaximum   = 1000.0; // Maximum Y-Axis
  public double       dYMinimum   = 1.0;    // Minimum Y-Axis
  public double       dYIncrement = 1.0;    // Increment along the Y-Axis
  public String       sYAxis      = "Production";  // Y-Axis Label.
  private String      sYIncr      = "Cycles:";

  // Data Curve Control Variables

  private static final int    _CUMULATIVE = 0;
  private static final int    _RAW        = 1;
  private static final int    _MEAN       = 2;
  private static final int    _MEDIAN     = 3;
  private static final int    _TOTAL      = 4;

  private static final String scb[] = {
	"Cumulative",
    "Raw Data",
    "Average",
    "Median" };

  private static final Color cCUMULTIVE = new Color(102,102,  0);
  private static final Color cRAW       = new Color(  0,180,  0);
  private static final Color cMEAN      = new Color(180,  0,  6);
  private static final Color cMEDIAN    = new Color(204,102,  0);

  private static final Color cbCOLOR[] = { cCUMULTIVE, cRAW, cMEAN, cMEDIAN };

  private static final int icb[] = { 0, 1, 0, 0 };

  private JCheckBox cb[]  = null;

  // Global Frame Widgets

  // .. Plot Titles

  private JTextField txtTitle1 = new JTextField();
  private JTextField txtTitle2 = new JTextField();
  private JTextField txtTitle3 = new JTextField();

  // -- File Button Menu

  private JButton btnPDF   = null; // Save Data as PNG with option to create PDF
  private JButton btnClose = null; // Close Dialog

  // -- Product Radio Buttons

  private JRadioButton rbProduct[]  = null;  // Product OIL or GAS

  // -- Limits Frame

  private int iScreenWidth  = 600;
  private JButton btnLimits = new JButton();
  private JFrame  fLimits   = null; // Limits Frame

  /** Constructor productionPlotFrame()
   *  <p> This is the constructor for this class
   * @param stData   = Earthquake Data Structure
   */

  public productionPlotFrame( productionListStruct st )
  {
	String sTitle  = "";
    double dMin    = 0.0;
    double dMax    = 0.0;
    int    iMin    = 0;
    int    iMax    = 0;

    try
    {
	  if (st != null)
	  {
		sTypes       = production.productionPlotUtility.getTypes( st, sTypes );
		iTypes       = production.productionPlotUtility.countTypes( sTypes );

        stDatas      = new productionPlotStruct[iTypes];
        for (int i=0; i<iTypes; i++)
        {
	      stDatas[i] = production.productionPlotUtility.getPlotData( st, sTypes[i] );
//production.productionPlotUtility.print(stDatas[i]);
//  System.out.println(i+" "+sTypes[i]);
        }

  	    stData       = production.productionPlotUtility.getPlotData( st, sTypes[iData] );
	  }

	  if (stData != null)
	  {
        this.tsStart = new Timestamp(stData.tsStart.getTime());
        this.tsEnd   = new Timestamp(stData.tsEnd.getTime());

        dMin = Math.log(stData.dMinimum)/Math.log(10.0);
        if (dMin < 0.0) { dMin = dMin - 1.0; }
        iMin = (int) dMin;
        dYMinimum = Math.pow(10.0, iMin);

        dMax = 1 + Math.log(stData.dMaximum)/Math.log(10.0);
        iMax = (int) dMax;

        iYCycles = iMax-iMin;
        iMax = iMin + iYCycles;
        dYMaximum = Math.pow(10.0, iMax);

        if ((sTypes[iData].equals("GAS")) || (sTypes[iData].equals("OIL")))
        {
          sTitle2 = new String(sTypes[iData] + " Production");
          sYAxis  = new String(sTypes[iData] + " Production ("+stData.sUnits+")");
		}
        else
        {
          sTitle2 = new String(sTypes[iData] + " Injection");
          sYAxis  = new String(sTypes[iData] + " Injection ("+stData.sUnits+")");
		}

        if (stData.sAPI.length() > 0)
          sTitle3 = new String(stData.sName +" [" + stData.sAPI + "]");
        else
          sTitle3 = new String(stData.sName);
	  }

      jbInit();

      addWindowListener( new productionPlotFrame_WindowListener() );
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
  }

  /** Method jbInit()
   *  <p> This method will create this Frame
   * @throws Exception
   */

  private void jbInit() throws Exception
  {
    int         iYs         = 0;
    int         iMs         = 0;
    int         iYe         = 0;
    int         iMe         = 0;

    int         iWidth      = 0;
    int         iHeight     = 0;

	JPanel      pnlTop      = new JPanel();
	JPanel      pnlMenu     = new JPanel();
	JPanel      pnlBTN      = new JPanel();

    JPanel      pnlProd     = new JPanel();

    JScrollPane jScrollPane = new JScrollPane();

    JButton     btnI        = new JButton();
    JButton     btnII       = new JButton();
    JButton     btnIII      = new JButton();
    JButton     btnIV       = new JButton();

    ButtonGroup groupProduct = new ButtonGroup();

    // Create an Observable

    notifier = new productionPlotFrameObservable();

    // Add this Dialog to the notifier as observer

    notifier.addObserver(this);

    // Build Panel

    this.getContentPane().setLayout(new BorderLayout());
    this.setTitle( sTitle );

    // Top Panel

    pnlTop.setLayout(new BorderLayout());

    // Menu Buttons

    pnlMenu.setLayout(new BorderLayout());

    btnClose = new JButton(createImageIcon(cmn.cmnStruct.CLOSE));
    btnClose.setPreferredSize(new Dimension(29, 29));
    btnClose.setToolTipText("Close Program");
    btnClose.addActionListener(this);

    // -- 1st Separator Button

    btnI.setPreferredSize(new Dimension(4, 35));
    btnI.setText("");

    // -- Save Data Buttons

    btnPDF = new JButton(createImageIcon(cmn.cmnStruct.PDF));
    btnPDF.setPreferredSize(new Dimension(29, 29));
    btnPDF.setToolTipText("Create PNG image of Production Plot");
    btnPDF.addActionListener(this);

    // -- 2nd Separator Button

    btnII.setPreferredSize(new Dimension(4, 35));
    btnII.setText("");

    // -- 3rd Separator Button

    btnIII.setPreferredSize(new Dimension(4, 35));
    btnIII.setText("");

    // Check boxes to turn curves on or off

    cb      = new JCheckBox[_TOTAL];

    for (int i=0; i<_TOTAL; i++)
    {
	  cb[i] = new JCheckBox();
	  cb[i].setFont(new java.awt.Font("Dialog", 1, 14));
	  cb[i].setText( scb[i] );
	  cb[i].setForeground( cbCOLOR[i] );
	  if (icb[i] == 1) { cb[i].setSelected(true); }
	  cb[i].addActionListener(this);
    }

    // Change Limits Button

    btnLimits.setFont(new java.awt.Font("Dialog", 1, 11));
    btnLimits.setText("Modify Limits");
    btnLimits.addActionListener(this);

    setDate( stData.tsStart );
    iYs = iYear;
    iMs = iMonth;
    tsStart = getTimestamp( iYs, iMs );

    setDate( stData.tsEnd );
    iYe = iYear;
    iMe = iMonth;

    iMe++;
	if (iMe > 12)
	{
	  iYe++;
	  iMe = 1;
	}

    tsEnd = getTimestamp( iYe, iMe );

    // -- 4th Separator Button

    btnIV.setPreferredSize(new Dimension(4, 35));
    btnIV.setText("");

    // -- Product Buttons

    if (iTypes > 0)
    {
      rbProduct = new JRadioButton[iTypes];

      for (int i=0; i<iTypes; i++)
      {
	    rbProduct[i] = new JRadioButton();
        rbProduct[i].setFont(new java.awt.Font("Dialog", 1, 12));
        rbProduct[i].setText(sTypes[i]);
        if (iData == i) rbProduct[i].setSelected(true);
        rbProduct[i].addActionListener(this);

        groupProduct.add(rbProduct[i]);
        pnlProd.add(rbProduct[i], null);
      }
	}

    // Center Panel

    panel = new productionPlot( stData, "", sYAxis,
                                tsStart, tsEnd, dYMinimum, dYMaximum, (double)iYCycles );

    iWidth  =  50 + panel.getPlotWidth();
    iHeight = 150 + panel.getPlotHeight();

    setTitle1(sTitle1);
    setTitle2(sTitle2);
    setTitle3(sTitle3);

    // Attach Widgets

    this.getContentPane().add(pnlTop,      BorderLayout.NORTH);
    pnlTop.add(pnlMenu,                    BorderLayout.NORTH);
    pnlMenu.add(pnlBTN,                    BorderLayout.WEST);
    pnlBTN.add(btnClose,                   null);
    pnlBTN.add(btnI,                       null);
    pnlBTN.add(btnPDF,                     null);
    pnlBTN.add(btnII,                      null);

    for (int i=0; i<_TOTAL; i++)
    {
	  pnlBTN.add( cb[i] );
	}

    pnlBTN.add(btnIII,                     null);
    pnlBTN.add(btnLimits,                  null);

    pnlBTN.add(btnIV,                      null);
    pnlBTN.add(pnlProd,                    null);

    this.getContentPane().add(jScrollPane, BorderLayout.CENTER);
    jScrollPane.getViewport().add(panel,   null);

    // Set Dimension of Frame and Display it.

//    this.setSize(new Dimension(900, 820));
    this.setSize(new Dimension(iWidth, iHeight));
    Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
//    this.setLocation( (d.width - this.getSize().width)/2,
    this.setLocation( (d.width - this.getSize().width) - 10,
                      (d.height - this.getSize().height)/2);

    this.setResizable(true);
    this.setVisible(true);
  }

  /** METHOD createImageIcon()
   *  <p> This Method will create the Frame for this class.
   *  @param path = the path to the image
   *  @return image icon
   */

  protected ImageIcon createImageIcon(String path)
  {
    java.net.URL imgURL = getClass().getResource(path);
    ImageIcon    image  = null;

    if (imgURL != null)
    {
      image = new ImageIcon(imgURL);
    }
    else
    {
       System.err.println("Couldn't find file: " + path);
    }

    return (image);
  }

  /* ================================================================== *
   * ------------------------- CLOSE METHODS -------------------------- *
   * ================================================================== */

  /** METHOD productionPlotFrame_WindowListener()
   *  <p> This Class will handle Frame Close Action using the X Button on Frame
   */

  public class productionPlotFrame_WindowListener extends WindowAdapter
  {
    public void windowClosing(WindowEvent e)
    {
	  close();
//	  if (notifier != null)
//        notifier.notifyObservers(new String("Close Plot Control"));
    }
  }

  /** Method close()
   *  <p> This method will deallocate the global variables to force Java
   *      to free allocated memory.
   */

  public void close()
  {
    stData   = null;

    for (int i=0; i<iTypes; i++)
    {
	  if (stDatas[i] != null)
        stDatas[i].delete();
      stDatas[i] = null;
    }
    stDatas = null;

    if (panel != null)
      panel.close();
    panel     = null;

    if (pLimits != null)
      pLimits.close();
    pLimits   = null;

    if (fLimits != null)
      fLimits.dispose();
    fLimits = null;

    if (pSearch != null)
      pSearch.close();
    pSearch   = null;

    tsStart   = null; // Starting Timestamp
    tsEnd     = null; // Ending Timestamp

    sPath     = null;
    sName     = null;

    sTitle    = null;
    sTitle1   = null;
    sTitle2   = null;
    sTitle3   = null;

    sXAxis    = null;
    sXIncr    = null;
    sYAxis    = null;
    sYIncr    = null;

    // Global Frame Widgets

	cb = null;

    // -- Menus Buttons

    btnPDF      = null;
    btnClose    = null;

    dispose();
  }

  /* ================================================================== *
   * -------------------------- GET METHODS --------------------------- *
   * ================================================================== */

  /** Method getImage()
   * <p> This method will convert this plot to a buffered image
   * @return image = profile plot buffered image
   */

  public BufferedImage getImage()
  {
    BufferedImage image = null;

    if (panel != null) image = panel.getImage();

    return (image);
  }

  /** Method getTimestamp()
   *  <p> This method will convert the Date Time String into a Timestamp.
   *  @param iY = Year
   *  @param iM = iMonth
   */

  private Timestamp getTimestamp( int iY, int iM )
  {
	Timestamp        ts    = null;
    SimpleDateFormat dfm   = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.SSSSSS");
    String           sText = "";
    long             ltime = (long) 0;


    sText = new String( iY + "-" + iM + "-" + "01 00:00:00.0");

    try
    {
      ltime = dfm.parse(sText).getTime();
      ts    = new Timestamp( ltime );
	}
    catch (ParseException e)
    {
      e.printStackTrace();
    }

	return (ts);
  }

  /* ================================================================== *
   * -------------------------- SET METHODS --------------------------- *
   * ================================================================== */

  /** Method setDate()
   *  <p> This method will set the Date Value into either the
   *      Start Date or End Date.
   *  @param ts    = timestamp
   */

  private void setDate(Timestamp ts)
  {
    int i = 0;
    String tokens[]    = null;
    String sDelimiter  = new String("[ ]+");
    String sDelimiter1 = new String("[-]+");
    String str         = "";
    String sDate       = "";
    String date[]      = { "", "", "" };
    int    iContinue   = 0;
    double d           = 0.0;
    int    id          = 0;

    if (ts != null)
    {
      str = new String(ts.toString());
      tokens = str.split( sDelimiter );
      if (tokens.length > 0)
        sDate = new String(tokens[0].trim());

      tokens = sDate.split( sDelimiter1 );
      if (tokens.length == 3)
      {
  	    for (i=0; i<3; i++)
	    {
	      date[i] = new String(tokens[i]);
	    }

        iYear  = cmn.cmnString.stringToInt(date[0]);
	    iMonth = cmn.cmnString.stringToInt(date[1]);
      }
    }
  }

  /** Method refresh()
   * <p> This method will refresh the plot if it exists
   */

  public void refresh() { panel.repaint(); }

  /** Method setCurve(int iCurve)
   *  <p> This method will set the Curves ON/OFF depending on selection
   */

  private void setCurve(int iCurve)
  {
	int iValue = -1;

	if (cb[iCurve].isSelected())
	  iValue = ON;
	else
	  iValue = OFF;

	if (iValue > -1)
	{
      switch (iCurve)
      {
        case _CUMULATIVE:
          panel.setCumulativeCurve(iValue);
          break;
        case _RAW:
          panel.setRawDataCurve(iValue);
          break;
        case _MEAN:
          panel.setAverageCurve(iValue);
          break;
        case _MEDIAN:
          panel.setMedianCurve(iValue);
          break;
	  }
	}
  }

  /** Method setLegend()
   *  <p> The Legend for the XY Plot.
   *  @param iLegend  - Number of Legend Rows.
   *  @param stLegend - Array of Legends.
   *

  public void setLegend(int iLegend, plotSymbolStruct stLegend[])
  {
    panel.setLegend(iLegend, stLegend);
  }
*/
  /** Method setFlag()
   *  <p> Set a Flag to filter extra data or not.
   *  @param bFlag = true to filter data or false not to filter extra data.
   *

  public void setFlag(boolean bFlag) { panel.setFlag(bFlag); }
*/
  /** Method setTitle1()
   * <p> This method will set the first user title
   * @param sText = the first title
   */

  public void setTitle1(String sText)
  {
	txtTitle1.setText(sText);
	panel.setTitle1(sText);
  }

  /** Method setTitle2()
   * <p> This method will set the second user title
   * @param sText = the second title
   */

  public void setTitle2(String sText)
  {
	txtTitle2.setText(sText);
	panel.setTitle2(sText);
  }

  /** Method setTitle3()
   * <p> This method will set the third user title
   * @param sText = the third title
   */

  public void setTitle3(String sText)
  {
	txtTitle3.setText(sText);
	panel.setTitle3(sText);
  }

  /** Method setXAxis()
   *  @param tsStart = starting Date & Time
   *  @param tsEnd   = ending Data & Time
   */

  public void setXAxis( Timestamp tsStart, Timestamp tsEnd )
  {
    panel.setXAxis(tsStart, tsEnd);
  }

  /** Method setYAxis()
   * <p> This method will reset the Y-Axis Parameters
   * @param sText = The Axis Label
   * @param iGrid = The type of grid to create
   * @param dMax  = The Maximum axis value
   * @param dMin  = The Minimum axis value
   * @param dInc  = The Increment value on the axis - Linear Grid only
   * @param iCycle = The Number of Cycles - Log Grid only
   */

  public void setYAxis(String sText,
                       int    iGrid,
                       double dMax,
                       double dMin,
                       double dInc,
                       int    iCycle)
  {
    panel.setYAxis(sText, iGrid, dMax, dMin, dInc, iCycle);
  }

  /** Method setData()
   *  <p> This method will set the data to be plotted
   *  @param iData = The data type to be plotted
   */

  public void setData(int iData)
  {
    double dMin    = 0.0;
    double dMax    = 0.0;
    int    iMin    = 0;
    int    iMax    = 0;

    int    iYs     = 0;
    int    iMs     = 0;
    int    iYe     = 0;
    int    iMe     = 0;

	this.iData = iData;

    if (panel != null)
    {
	  if (stData != null)
	    stData.delete();
	  stData = null;

  	  stData  = production.productionPlotUtility.copy( stDatas[iData] );

	  if (stData != null)
	  {
        tsStart = new Timestamp(stData.tsStart.getTime());
        tsEnd   = new Timestamp(stData.tsEnd.getTime());

        setDate( stData.tsStart );
        iYs = iYear;
        iMs = iMonth;
        tsStart = getTimestamp( iYs, iMs );

        setDate( stData.tsEnd );
        iYe = iYear;
        iMe = iMonth;

        iMe++;
	    if (iMe > 12)
	    {
	      iYe++;
	      iMe = 1;
	    }

        tsEnd = getTimestamp( iYe, iMe );

        dMin = Math.log(stData.dMinimum)/Math.log(10.0);
        if (dMin < 0.0) { dMin = dMin - 1.0; }
        iMin = (int) dMin;
        dYMinimum = Math.pow(10.0, iMin);

        dMax = 1 + Math.log(stData.dMaximum)/Math.log(10.0);
        iMax = (int) dMax;

        iYCycles = iMax-iMin;

        iMax = iMin + iYCycles;
        dYMaximum = Math.pow(10.0, iMax);

        if ((sTypes[iData].equals("GAS")) || (sTypes[iData].equals("OIL")))
        {
          sTitle2 = new String(sTypes[iData] + " Production");
          sYAxis  = new String(sTypes[iData] + " Production ("+stData.sUnits+")");
		}
        else
        {
          sTitle2 = new String(sTypes[iData] + " Injection");
          sYAxis  = new String(sTypes[iData] + " Injection ("+stData.sUnits+")");
		}

//        sTitle3 = new String(stData.sName +" [" + sTypes[iData] + "]");
//        sYAxis  = new String("Production ("+stData.sUnits+")");

        panel.setData( stData, "", sYAxis,
                       tsStart, tsEnd, dYMinimum, dYMaximum, (double)iYCycles );
        setTitle2(sTitle2);
//        setTitle3(sTitle3);
	  }
	}
  }

  /* ===================================================================== *
   * ---------------------- LIMITS FRAME METHODS ------------------------- *
   * ===================================================================== */

  /** Method limitsFrame()
   *  <p> This method will create a Limits Frame to hold the limits Panel
   */

  private void limitsFrame()
  {
	int iX = 0;
	int iY = 0;

	if (pLimits != null)
	{
      fLimits = new JFrame();

      fLimits.setTitle("Limits");
      fLimits.getContentPane().add(pLimits, null);

      // Set the Size and Display the Frame

      Dimension d = Toolkit.getDefaultToolkit().getScreenSize();

      fLimits.setSize(350, 420);
      iX = iScreenWidth;
      iY = d.height - fLimits.getSize().height - 75;
      fLimits.setLocation(iX, 10); //iY);
//      fLimits.setLocation((d.width  - fLimits.getSize().width)/2,
//                          (d.height - fLimits.getSize().height)/2);

      fLimits.setResizable(true);
      fLimits.setVisible(true);

      fLimits.addWindowListener( new limitsFrame_WindowListener() );
	}
  }

  /** METHOD limitsFrame_WindowListener()
   *  <p> This Class will handle Frame Close Action using the X Button on Frame
   */

  public class limitsFrame_WindowListener extends WindowAdapter
  {
    public void windowClosing(WindowEvent e)
    {
	  pLimits.close();
      fLimits.dispose();
    }
  }

  /* ===================================================================== *
   * -------------------------- PDF ACTIONS ------------------------------ *
   * ===================================================================== */

  /** Method createPDF()
   *  <p> This method will create a PDF Image of Production Plot
   */

  private void createPDF()
  {
    production.io.productionIO_Files.createImage( sPath, sName, panel.getImage() );
    production.io.productionIO_Files.createHTML( sPath, sName );
  }

  /* ===================================================================== *
   * ---------------------------- ACTIONS -------------------------------- *
   * ===================================================================== */

  /** METHOD actionPerformed()
   *  <p> This Method will handle all the actions within the Frame.
   *  @param event - Action Event
   */

  public void actionPerformed(ActionEvent event)
  {
    if (event.getSource() == btnPDF) // Create a PDF of Profile Plot
    {
	  if (sPath.length() == 0)
        sPath = production.io.productionIO_Files.getHomeDir();

      if (sName.length() == 0)
        sName = new String( "Offset_Gas" );

      pSearch = new guiSearchDirectoryFrame(notifier, sPath, sName, ".png");
    }

    for (int i=0; i<_TOTAL; i++)
    {
	  if (event.getSource() == cb[i]) { setCurve(i); }
	}

    if (event.getSource() == btnLimits)
    {
	  if (pLimits != null)
	    pLimits.close();
	  pLimits = null;

	  if (fLimits != null)
	    fLimits.dispose();
	  fLimits = null;

	  pLimits = new productionLimitsPanel( notifier,
		                                   sTitle1,  sTitle2,  sTitle3,
                                           tsStart, tsEnd,
                                           sYAxis,
                                           dYMaximum,  dYMinimum,  iYCycles );
	  limitsFrame();
	}

	for (int i=0; i<iTypes; i++)
	{
	  if (event.getSource() == rbProduct[i])
	  {
        setData(i);
	  }
	}

    if (event.getSource() == btnClose)  { close(); }
  }

  /* ===================================================================== *
   * ------------------------ OBSERVER ACTIONS --------------------------- *
   * ===================================================================== */

  /** Method update()
   * <p> This method will handle the Observable Actions
   * @param obj = The Observable making the call
   * @param arg = The returned object
   */

  public void update(Observable obj, Object arg)
  {
    String    sArg = new String( (String) arg);
    Timestamp ts   = null;

    if (sArg.equals("GET TITLE 1"))
    {
      setTitle1(pLimits.getTitle(1));
	}

    if (sArg.equals("GET TITLE 2"))
    {
      setTitle1(pLimits.getTitle(2));
	}

    if (sArg.equals("GET TITLE 3"))
    {
      setTitle1(pLimits.getTitle(3));
	}

    if (sArg.equals("GET X"))
    {
	  ts = pLimits.getTime(_START);            // Start Date & Time
	  if (ts != null)
	  {
        tsStart = new Timestamp(ts.getTime());
	  }

      ts = pLimits.getTime(_END);              // End Date & Time
      if (ts != null)
      {
        tsEnd = new Timestamp(ts.getTime());
	  }

      setXAxis( tsStart, tsEnd );
	}

    if (sArg.equals("GET Y"))
    {
      sYAxis = new String( pLimits.getYAxis()); // Y-Axis Label
      dYMaximum = pLimits.getYMax();            // Maximum Y-Axis
      dYMinimum = pLimits.getYMin();            // Minimum Y-Axis
      iYCycles  = pLimits.getYCycle();          // Number of Log Cycles (Only of Log Scales)

      setYAxis( sYAxis, iYLog, dYMaximum, dYMinimum, dYIncrement, iYCycles );
	}

    if (sArg.equals("Directory Path Selected"))
    {
      if (pSearch != null)
      {
        sPath = new String( pSearch.getPath() );
        sName = new String( pSearch.getFile() );
      }

      createPDF();

      if (pSearch != null)
        pSearch.close();
      pSearch = null;
	}
  }
}

/** CLASS productionPlotFrameObservable()
  *  <p> This Class will handle Observable Actions when action occurs on a
  *      frame or panel
  */

class productionPlotFrameObservable extends Observable
{
  public void notifyObservers(Object b)
  {
    setChanged();
    super.notifyObservers(b);
  }
}

/*
 *  @version 1.1 03/31/206
 *  @author  John Victorine
 */
