/*
 * @productionPlotUtility.java Version 1.1 03/30/2016
 *
 * Copyright (c) 2016 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package production;

import java.sql.Timestamp;
import java.sql.Date;
import java.text.SimpleDateFormat;
import java.text.ParseException;

import production.productionPlotStruct;  // Production Plot Data Structure
import production.productionListStruct;  // Production List Structure

/** Class productionPlotUtility
 *  <p> This is the production data structure utility class.
 *
 *  @version 1.1 03/30/2016
 *  @author  John R. Victorine
 */

public class productionPlotUtility
{
  /** Method productionPlotStruct copy(productionPlotStruct stOld)
   *  <p> This method will copy the seismic data structure
   *  @param  stOld = Old production data structure
   *  @return stNew = New production data structure
   */

  public static productionPlotStruct copy(productionPlotStruct stOld)
  {
	productionPlotStruct stNew = null;

	if (stOld != null)
	{
	  stNew            = new productionPlotStruct();

      stNew.sKID       = new String(stOld.sKID);      // Production Data Primary Key
      stNew.sID        = new String(stOld.sID);       // Other ID for the Production Data
      stNew.sProduct   = new String(stOld.sProduct);  // Type of Production Data
      stNew.sName      = new String(stOld.sName);     // Name of Production Data
      stNew.sAPI       = new String(stOld.sAPI);      // API-Number

      stNew.sApp       = new String(stOld.sApp);      // URL retrieving Data
      stNew.sBy        = new String(stOld.sBy);       // Search By

      stNew.sUnits     = new String(stOld.sUnits);    // Units

      stNew.tsStart    = new Timestamp(stOld.tsStart.getTime());
      stNew.tsEnd      = new Timestamp(stOld.tsEnd.getTime());

      stNew.dMaximum   = stOld.dMaximum;             // Maximum Data Value
      stNew.dMinimum   = stOld.dMinimum;             // Minimum Data Value

      // Plot Dimensions

      stNew.iWidth     = stOld.iWidth;  // Actual Plotting Width
      stNew.iHeight    = stOld.iHeight; // Actual plotting Height

      // Records

      stNew.iRows      = stOld.iRows;                // Total Number of rows
      stNew.ts         = new Timestamp[stNew.iRows]; // Time Stamp of data records
      stNew.dRaw       = new double[stNew.iRows];    // Raw Data records
      stNew.dAvg       = new double[stNew.iRows];    // Average Raw Data records
      stNew.dMedian    = new double[stNew.iRows];    // Median Raw Data records
      stNew.dCUM       = new double[stNew.iRows];    // Cumulative
      stNew.sType      = new String[stNew.iRows];    // Type of Data record

	  if (stNew.iRows > 0)
	  {
		for (int i=0; i<stNew.iRows; i++)
		{
		  stNew.ts[i]         = new Timestamp(stOld.ts[i].getTime());
		  stNew.dRaw[i]       = stOld.dRaw[i];
		  stNew.dAvg[i]       = stOld.dAvg[i];
		  stNew.dMedian[i]    = stOld.dMedian[i];
		  stNew.dCUM[i]       = stOld.dCUM[i];
		  stNew.sType[i]      = new String(stOld.sType[i]);
		}
	  }
	}

	return (stNew);
  }

  /** Method getTypes(productionPlotStruct stOld)
   *  <p> This method will copy the seismic data structure
   *  @param  stList = production List structure
   *  @param  sTypes = the types of data that is in the production List Structure
   *  @return stPlot = production plot data structure
   */

  public static String[] getTypes( productionListStruct stList, String sTypes[] )
  {
	int    icnt  = 0;
	String sType = "";
	int    icont = 0;

	if ((stList != null) && (sTypes != null))
	{
      for (int i=0; i<stList.iCount; i++)
      {
		if (!stList.stItem[i].sProduct.equals(sType))
		{
		  if (icnt < sTypes.length)
		  {
			icont = 0;
			for (int j=0; j<sTypes.length; j++)
			{
			  if (stList.stItem[i].sProduct.equals(sTypes[j]))
			  {
				icont = 1;
			  }
			}

			if (icont == 0)
		    {
		      sType        = new String( stList.stItem[i].sProduct );
		      sTypes[icnt] = new String( stList.stItem[i].sProduct );
		      icnt++;
		    }
		  }
		}
	  }
	}

	return (sTypes);
  }

  /** Method productionPlotStruct countTypes()
   *  <p> This method will copy the seismic data structure
   *  @param  sTypes = the types of data that is in the production List Structure
   *  @return iTypes = total number of types greater than 0 length
   */

  public static int countTypes(String sTypes[])
  {
	int iTypes = 0;

	if (sTypes != null)
	{
	  for (int i=0; i<sTypes.length; i++)
	  {
		if (sTypes[i].length() > 0)
		{
		  iTypes++;
		}
	  }
	}

	return (iTypes);
  }

  /** Method productionPlotStruct getPlotData(productionListStruct stList)
   *  <p> This method will copy the seismic data structure
   *  @param  stList   = production list structure
   *  @param  sProduct = Type of Production to extract
   *  @return stPlot   = production plot data structure
   */

  public static productionPlotStruct getPlotData(productionListStruct stList, String sProduct)
  {
	productionPlotStruct stPlot = null;
	int                  iCount = 0;
    double               dSum   = 0.0;
    int                  iStart = 0;
    int                  iNew   = 1;
    int                  iTotal = 0;

	if (stList != null)
	{
	  stPlot          = new productionPlotStruct();

      stPlot.sKID     = new String(stList.sKID);      // Production Data Primary Key
      stPlot.sID      = new String(stList.sID);       // Other ID for the Production Data
      stPlot.sProduct = new String(stList.sType);     // Type of Production Data
      stPlot.sName    = new String(stList.sName);     // Name of Production Data
      stPlot.sAPI     = new String(stList.sAPI);      // API-Number

      stPlot.sApp     = new String(stList.sApp);      // URL retrieving Data
      stPlot.sBy      = new String(stList.sBy);       // Search By

      stPlot.sUnits   = new String("");               // Units

      stPlot.ts       = new Timestamp[stList.iCount]; // Time Stamp of data record
      stPlot.dRaw     = new double[stList.iCount];    // Raw Data record
      stPlot.dAvg     = new double[stList.iCount];    // Average Raw Data record
      stPlot.dMedian  = new double[stList.iCount];    // Median Raw Data record
      stPlot.dCUM     = new double[stList.iCount];    // Cumulative Data record
      stPlot.sType    = new String[stList.iCount];    // Data type

      iTotal = 0;
      for (int i=0; i<stList.iCount; i++)
      {
        if (sProduct.equals(stList.stItem[i].sProduct))
        {
		  iTotal++;
  		  if (iStart==0)
		  {
            stPlot.sUnits   = new String(stList.stItem[i].sUnits);   // Units
            iStart++;
            iNew = 0;
		  }

          if (iCount < stList.iCount)
  	      {
            stPlot.ts[iCount]    = getTimeStamp( stList.stItem[i].sDate );
            stPlot.dRaw[iCount]  = stList.stItem[i].data; // production Data
            dSum                 = dSum + stPlot.dRaw[iCount];
            stPlot.dAvg[iCount]  = dSum / iTotal;

            if (i > 2)
	          stPlot.dMedian[iCount] = computePercentile(iCount, 0.5,  stPlot.dRaw);
            else
              stPlot.dMedian[iCount] = stPlot.dAvg[iCount];

            stPlot.dCUM[iCount]  = dSum;
            stPlot.sType[iCount] = new String(stList.stItem[i].sProduct);

            if (iNew==0)
            {
              stPlot.dMaximum = stPlot.dRaw[iCount]; // Maximum Data Value
              stPlot.dMinimum = stPlot.dRaw[iCount]; // Minimum Data Value
		      stPlot.tsStart  = new Timestamp(stPlot.ts[iCount].getTime()); // Starting Date
		      stPlot.tsEnd    = new Timestamp(stPlot.ts[iCount].getTime()); // Ending Date
		      iNew = 1;
		    }
		    else
		    {
		      stPlot.tsEnd    = new Timestamp(stPlot.ts[iCount].getTime()); // Ending Date
//System.out.println(iCount+" "+stPlot.sUnits+" "+stPlot.tsEnd.toString());
		    }

		    if (stPlot.dMinimum > stPlot.dRaw[iCount])
		      stPlot.dMinimum = stPlot.dRaw[iCount];

		    if (stPlot.dMaximum < stPlot.dRaw[iCount])
		      stPlot.dMaximum = stPlot.dRaw[iCount];
//if (stPlot.ts[iCount] != null)
//System.out.println(iCount+" "+stPlot.ts[iCount].toString()+" "+stPlot.dRaw[iCount]);
		    iCount++;
	      }
		}
	  }
//System.out.println(iCount+" "+stPlot.dMinimum+" "+stPlot.dMaximum+" "+
//                   stPlot.tsStart.toString()+" "+stPlot.tsEnd.toString());
	  stPlot.iRows = iCount;
	}

	return (stPlot);
  }

  /** Method getTimeStamp( String str )
   *  <p> This method will get the Time Stamp from the Production Data Time.
   *      The Production ORACLE Data Time with Format "MM/DD/YYYY HH24:MI:SS".
   *  @param  sDate = The Production ORACLE Data Time
   *  @return ts    = Timestamp with the Format "yyyy-MM-dd HH:mm:ss.SSSSSS".
   */

   public static Timestamp getTimeStamp( String stng )
   {
	 Timestamp ts = null;
     int    i = 0;
     String tokens[]    = null;
     String str         = "";
     String sDelimiter  = new String("[ ]+");
     String sDelimiter1 = new String("[/]+");
     String sDelimiter2 = new String("[:]+");
     String sDate       = "";
     String sTime       = "";
     String date[]      = { "", "", "" };
     String time[]      = { "", "", "" };
     int    iContinue   = 0;
     double d           = 0.0;
     int    id          = 0;

     SimpleDateFormat dfm   = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.SSSSSS");
     String           sText = "";
     long             ltime = (long) 0;

	 if (stng != null)
	 {
	   if (stng.length() > 0)
	   {
		 str    = new String(stng);
         tokens = str.split( sDelimiter );
         if (tokens.length > 0)
           sDate = new String(tokens[0].trim());

         tokens = sDate.split( sDelimiter1 );
         if (tokens.length == 3)
         {
  	       for (i=0; i<3; i++)
	       {
	         date[i] = new String(tokens[i]);
	       }

	       iContinue = 1;

           if (iContinue == 1)
	       {
	         iContinue = 0;

   		     str    = new String(stng);
	         tokens = str.split( sDelimiter );
	         if (tokens.length > 1)
	           sTime = new String(tokens[1].trim());

             tokens = sTime.split( sDelimiter2 );
             if (tokens.length == 3)
             {
  	           for (i=0; i<3; i++)
  	           {
	             time[i] = new String(tokens[i]);
	           }

	           iContinue = 1;

	           if (iContinue == 1)
	           {

                 sText = new String( date[2] + "-" +  // Year
  		                             date[0] + "-" +  // Month
		                             date[1] + " " +  // Day
		                             time[0] + ":" +  // 24 Hour
		                             time[1] + ":" +  // Minute
		                             time[2] + ".0"); // Seconds

                 try
                 {
                   ltime = dfm.parse(sText).getTime();
                   ts    = new Timestamp( ltime );
	             }
                 catch (ParseException e)
                 {
                   e.printStackTrace();
                 }
			   }
			 }
		   }
		 }
	   }
	 }

	 return (ts);
  }

  /** Method computePercentile()
   *  <p> This method will compute the Percentile value
   *  @param iTotal   = the Total number of rows
   *  @param dPercent = the decimal percentage value
   *  @param data     = array of numbers presorted
   */

  public static double computePercentile(
	                     int iTotal, double dPercent, double data[])
  {
	double dValue    = 0.0;
	double RowNum    = 0.0;
	double CelRowNum = 0.0;
	double FlrRowNum = 0.0;
	int    iFlr      = 0;
	int    iCel      = 0;

	if ((iTotal > 0) && data != null)
	{
	  RowNum    = 1.0 + (dPercent * (iTotal-1));
	  CelRowNum = Math.ceil(RowNum);
	  FlrRowNum = Math.floor(RowNum);

	  if ((CelRowNum == RowNum) && (FlrRowNum == RowNum))
	  {
		if (RowNum < data.length)
	      dValue = data[(int) RowNum];
	  }
	  else
	  {
		iFlr = (int) FlrRowNum;
		if (iFlr >= iTotal)
		  iFlr = iTotal-1;

		iCel = (int) CelRowNum;
		if (iCel >= iTotal)
		  iCel = iTotal-1;

		if ((iFlr < data.length) && (iCel < data.length))
		{
  	      dValue = (CelRowNum-RowNum) * data[iFlr] +
	               (RowNum-FlrRowNum) * data[iCel];
		}
	  }
	}

	return (dValue);
  }

  /** Method bubbleSort()
   * <p> This method will sort in ascending depth order (lowest to highest)
   * @param  data = data array to be sorted
   * @return data = sorted data array
   */

  public static double[] bubbleSort(double data[])
  {
    boolean swappedOnPrevRun = true;
    double  dTemp   = 0.0;

    if (data != null)
    {
      if (data.length > 1)
      {
        while(swappedOnPrevRun)
        {
          // this variable keeps track of whether to continue sorting or exit

          swappedOnPrevRun = false;

          // loop through every element in the array, except for the last one

          for(int i=0; i<data.length-1; i++)
          {
            // if current element is greater than the next swap the two elements

            if(data[i] > data[i+1])
            {
              // we don't want the loop to end just yet, we're not done

              swappedOnPrevRun = true;

              // store element i in a temporary variable

              dTemp = data[i];

              // set element i+1 to where i used to be

              data[i] = data[i+1];

              // release the old i from temp into i+1 slot

              data[i+1] = dTemp;
            }
          }
        }
      }
    }

    return (data);
  }

  /** Method print()
   *  <p> This method will print the production Data Structure
   */

  public static void print(productionPlotStruct st)
  {
    if (st != null)
    {
      for (int i=0; i<st.iRows; i++)
      {
        System.out.println(st.ts[i].toString()+" "+st.dRaw[i]+" "+st.sType[i]);
      }

      System.out.print("Min="+st.dMinimum+" Max="+st.dMaximum);
      System.out.println(" Start="+st.tsStart.toString()+" End="+st.tsEnd.toString());
    }
  }
}

/*
 *  @version 1.1 03/30/2016
 *  @author  John Victorine
 */
